@@ -6,7 +6,7 @@
 **
 ** (see the file 'base_main.php' for license details)
 **
-** Project Leads: Kevin Johnson <kjohnson@secureideas.net>, Joel Esler <joelesler@users.sourceforge.net>
+** Project Leads: Kevin Johnson <kjohnson@secureideas.net>
 ** Built upon work by Roman Danyliw <rdd@cert.org>, <roman@danyliw.com>
 **
 ** Purpose: Common Functions
@@ -49,6 +49,26 @@ function GetSensorName($sid, $db)
     return $name;
 }
 
+function GetVendor($mac)
+{
+    $mac = str_replace(":", "", $mac);
+    $mac = substr($mac, 0, 6);
+    $vendor = 'unknown';
+
+    if (@$fp = fopen("base_mac_prefixes.map", "r")) {
+       while (!feof($fp)) {
+           $line = fgets($fp);
+           if (strcmp($mac, substr($line, 0, 6)) == 0)
+               $vendor = substr($line, 7, strlen($line)-8);
+       }
+       fclose($fp);
+    } 
+    else 
+       return "can't open vendor map";
+
+    return $vendor;
+}
+
 function InputSafeSQL (&$SQLstr)
 /* Removes the escape sequence of \' => ' which arise when a variable containing a '-character is passed
    through a POST query.  This is needed since otherwise the MySQL parser complains */
@@ -287,29 +307,59 @@ function TCPOption2str($tcpopt_code)
    switch($tcpopt_code)
    {
       case 2:                  /* TCPOPT_MAXSEG - maximum segment*/ 
-          return "MSS";
+          return "(2) MSS";
       case 0:                  /* TCPOPT_EOL */
-          return "EOL";
+          return "(0) EOL";
       case 1:                  /* TCPOPT_NOP */
-          return "NOP";
+          return "(1) NOP";
       case 3:                  /* TCPOPT_WSCALE (rfc1072)- window scale factor */
-          return "WS";
+          return "(3) WS";
       case 5:                  /* TCPOPT_SACK (rfc1072)- selective ACK */
-          return "SACK";
+          return "(5) SACK";
       case 4:                  /* TCPOPT_SACKOK (rfc1072)- selective ACK OK */
-          return "SACKOK";
+          return "(4) SACKOK";
       case 6:                  /* TCPOPT_ECHO (rfc1072)- echo */
-          return "Echo";
+          return "(6) Echo";
       case 7:                  /* TCPOPT_ECHOREPLY (rfc1072)- echo reply */
-          return "Echo Reply";
+          return "(7) Echo Reply";
       case 8:                  /* TCPOPT_TIMESTAMP (rfc1323)- timestamps */
-          return "TS";
+          return "(8) TS";
+      case 9:                  /* RFC1693 */
+          return "(9) Partial Order Connection Permitted";
+      case 10:                  /* RFC1693 */ 
+          return "(10) Partial Order Service Profile";
       case 11:                 /* TCPOPT_CC (rfc1644)- CC options */
-          return "CC";
+          return "(11) CC";
       case 12:                 /* TCPOPT_CCNEW (rfc1644)- CC options */
-          return "CCNEW";
+          return "(12) CCNEW";
       case 13:                 /* TCPOPT_CCECHO (rfc1644)- CC options */
-          return "CCECHO";
+          return "(13) CCECHO";
+      case 14:                 /* RFC1146 */
+          return "(14) TCP Alternate Checksum Request";
+      case 15:                 /* RFC1146 */
+          return "(15) TCP Alternate Checksum Data";
+      case 16:
+          return "(16) Skeeter";
+      case 17:
+          return "(17) Bubba";
+      case 18:                 /* Subbu and Monroe */
+          return "(18) Trailer Checksum Option";
+      case 19:                 /* Subbu and Monroe */
+          return "(19) MD5 Signature";
+      case 20:                 /* Scott */
+          return "(20) SCPS Capabilities";
+      case 21:                /* Scott */
+          return "(21) Selective Negative Acknowledgements";
+      case 22:                /* Scott */
+          return "(22) Record Boundaries";
+      case 23:                /* Scott */
+          return "(23) Corruption Experienced";
+      case 24:                /* Sukonnik */
+          return "(24) SNAP";
+      case 25:
+          return "(25) Unassigned";
+      case 26:                /* Bellovin */
+          return "(26) TCP Compression Filter";
       default:
           return $tcpopt_code;
    }
@@ -374,6 +424,50 @@ function ICMPType2str($icmp_type)
           return "Address Mask Request";
       case 18:                            /* ICMP_ADDRESSREPLY */
           return "Address Mask Reply";
+      case 19:
+          return "Reserved (security)";
+      case 20:
+          return "Reserved (robustness)";
+      case 21:
+          return "Reserved (robustness)";
+      case 22:
+          return "Reserved (robustness)";
+      case 23:
+          return "Reserved (robustness)";
+      case 24:
+          return "Reserved (robustness)";
+      case 25:
+          return "Reserved (robustness)";
+      case 26:
+          return "Reserved (robustness)";
+      case 27:
+          return "Reserved (robustness)";
+      case 28:
+          return "Reserved (robustness)";
+      case 29:
+          return "Reserved (robustness)";
+      case 30:
+          return "Traceroute";
+      case 31:
+          return "Datagram Conversion Error";
+      case 32:
+          return "Mobile Host Redirect";
+      case 33:
+          return "IPv6 Where-Are-You";
+      case 34:
+          return "IPv6 I-Am-Here";
+      case 35:
+          return "Mobile Registration Request";
+      case 36:
+          return "Mobile Registration Reply";
+      case 37:
+          return "Domain Name Request";
+      case 38:
+          return "Domain Name Reply";
+      case 39:
+          return "SKIP";
+      case 40:
+          return "Photuris";
       default:
           return $icmp_type;
   }
@@ -420,7 +514,81 @@ function ICMPCode2str($icmp_type, $icmp_code)
         default:
             return $icmp_code;
      }
-  }
+  } 
+  elseif ( $icmp_type == 5 )
+  {
+     switch ($icmp_code)
+     {
+        case 0:
+            return "Redirect datagram for network/subnet";
+        case 1:
+            return "Redirect datagram for host";
+        case 2:
+            return "Redirect datagram for ToS and network";
+        case 3:
+            return "Redirect datagram for Tos and host";
+        default:
+            return $icmp_code;
+      }
+   }
+   elseif ( $icmp_type == 9 ) 
+   {
+      switch ($icmp_code)
+      {
+         case 0:
+             return "Normal router advertisement";
+         case 16:
+             return "Does not route common traffic";
+         default:
+             return $icmp_code;
+      }
+   }
+   elseif ( $icmp_type == 11 )
+   {
+      switch ($icmp_code)
+      {
+         case 0:
+             return "TTL exceeded in transit";
+         case 1:
+             return "Fragment reassembly time exceeded";
+         default:
+             return $icmp_code;
+      }
+   }
+   elseif ( $icmp_type == 12 ) 
+   {
+      switch ($icmp_code)
+      {
+         case 0:
+             return "Pointer indicates error";
+         case 1:
+             return "Missing a required option";
+         case 2:
+             return "Bad length";
+         default:
+             return $icmp_code;
+      }
+   }
+   elseif ( $icmp_type == 40 )
+   {
+      switch ($icmp_code)
+      {
+         case 0:
+            return "Bad SPI";
+         case 1:
+            return "Authentication failed";
+         case 2:
+            return "Decompression failed";
+         case 3:
+            return "Decryption failed";
+         case 4:
+            return "Need authentication";
+         case 5:
+            return "Need authorization";
+         default:
+             return $icmp_code;
+      }
+   }
   else
      return $icmp_code;
 }
@@ -535,41 +703,34 @@ function PrintHexPacketPayload ( $encoded_payload, $output_type )
 }
 
 // ************************************************************************************
-function PrintCleanHexPacketPayload ( $encoded_payload, $output_type )
+function PrintCleanHexPacketPayload( $encoded_payload, $output_type )
 {
-     /* strip out the <CR> at the end of each block */
-     $payload = str_replace("\n", "", $encoded_payload);
+     $len = strlen($encoded_payload);
 
-     $len = strlen($payload);
-
-     for ($i = 0; $i < strlen($payload); $i += 2 )
+     for ($i = 0; $i < $len; $i += 2 )
      {
-          if ( $i % 32 == 0 )
-          {
-             /* dump the ASCII characters */
-             if ( $i != 0 )
-             {
-                for ($j = $i-32; $j < $i; $j+=2 )
-                {
-                    $t = hexdec($payload[$j].$payload[$j+1]);
-                    $s = $s.PrintCleanPayloadChar($t, $output_type);
-                }
-             }
-          }
+         /* dump the ASCII characters */
+         $t = hexdec($encoded_payload[$i].$encoded_payload[$i+1]);
+         $s_tmp = PrintCleanPayloadChar($t, $output_type);
+
+         /* Join together several sequential non-ASCII characters displaying their count
+          * in one line. It makes easyer to look through payload in plain display mode.
+          * If current character is '<br>' and this is not last character of payload
+          * increment counter, else output non-ASCII count and flush counter.
+          */
+         if ( ($s_tmp == '<br>') && !($i+2 == $len) ) {
+             $count++;
+         } else {
+             if ($count > 1)
+                 $s .= '<DIV class="nonascii">['. $count .' non-ASCII characters]</DIV>';
+             elseif ($count == 1)
+                 $s .= '<br>';
+             $s .= $s_tmp;
+             $count = 0;
+         }
+
      }
 
-     /* print the ASCII decode */
-     if ( $i % 32 )
-        $start = $len - ($i % 32);
-     else
-        $start = $len - 32;
-
-     for ( $j = $start; $j < $i; $j+=2 )
-     {
-        $t = hexdec($payload[$j].$payload[$j+1]);
-        $s = $s.PrintCleanPayloadChar($t, $output_type);
-     } 
-
      return $s;
 }
 
@@ -598,7 +759,7 @@ function PrintPacketPayload($data, $encode_type, $output_type)
          $payload = PrintBase64PacketPayload($data, $output_type);
      else if ($encode_type == "0" )
      {
-         if ($_GET[asciiclean] == 1 or $_COOKIE[asciiclean] == "clean")
+         if ( ($_GET[asciiclean] == 1) || ( ($_COOKIE[asciiclean] == "clean") && (!isset($_GET[asciiclean])) ) )
 	 {
 	    // Print clean ascii display
             $payload = PrintCleanHexPacketPayload($data, $output_type);
@@ -632,9 +793,10 @@ function GetQueryResultID($submit, &$seq, &$sid, &$cid)
   $submit = str_replace("(", "", $submit);
   $submit = str_replace(")", "", $submit);
   $tmp = explode("-", $submit);
-  $seq = $tmp[0];
-  $sid = $tmp[1];
-  $cid = $tmp[2];
+  /* Since the submit variable is not cleaned do so here: */
+  $seq = CleanVariable($tmp[0], VAR_DIGIT);
+  $sid = CleanVariable($tmp[1], VAR_DIGIT);
+  $cid = CleanVariable($tmp[2], VAR_DIGIT);
 }
 
 function ExportPacket($sid, $cid, $db)
