@@ -6,7 +6,7 @@
 **
 ** (see the file 'base_main.php' for license details)
 **
-** Project Leads: Kevin Johnson <kjohnson@secureideas.net>, Joel Esler <joelesler@users.sourceforge.net>
+** Project Leads: Kevin Johnson <kjohnson@secureideas.net>
 ** Built upon work by Roman Danyliw <rdd@cert.org>, <roman@danyliw.com>
 **
 ** Purpose: displays a single alert   
@@ -43,7 +43,7 @@
     setcookie('asciiclean', 'clean');
   }
   
-  if ($_GET[asciiclean] == 0)
+  if ( isset($_GET[asciiclean]) && ($_GET[asciiclean] == 0) )
   {
     setcookie('asciiclean', 'normal');
   }
@@ -52,33 +52,54 @@
 function PrintCleanURL()
 {
   // This function creates the url to display the cleaned up payload -- Kevin
-  // This entire thing needs to be re-thought.
-  if ($_GET[asciiclean] == 1 or $_COOKIE["asciiclean"] == "clean")
+  $query = CleanVariable($_SERVER["QUERY_STRING"], VAR_PERIOD | VAR_DIGIT | VAR_PUNC | VAR_LETTER);
+  $sort_order=ImportHTTPVar("sort_order", VAR_LETTER | VAR_USCORE);
+
+  if ( ($_GET[asciiclean] == 1) || ( ($_COOKIE["asciiclean"] == "clean") && (!isset($_GET[asciiclean])) ) )
   {
     //create link to non-cleaned payload display
-    $query = $_SERVER["QUERY_STRING"];
-    $url = '<center><a href="base_qry_alert.php?'.$_SERVER["QUERY_STRING"].'&asciiclean=0">'._QANORMALD.'</a></center>';
-    //$_COOKIE["asciiclean"] == "clean";
+    $url = '<center><a href="base_qry_alert.php?'.$query;
+    $url.= '&sort_order='.$sort_order.'&asciiclean=0">'._QANORMALD.'</a></center>';
     return $url;
   }
   else
   {
     //create link to cleaned payload display
-    $query = $_SERVER["QUERY_STRING"];
-    $url = '<center><a href="base_qry_alert.php?'.$_SERVER["QUERY_STRING"].'&asciiclean=1">'._QAPLAIND.'</a></center>';
-    //$_COOKIE["asciiclean"] == "normal";
+    $url = '<center><a href="base_qry_alert.php?'.$query;
+    $url.= '&sort_order='.$sort_order.'&asciiclean=1">'._QAPLAIND.'</a></center>';
     return $url;
   }
   
 }
-// xxx jl
+
 function PrintBinDownload($db, $cid, $sid){
 // Offering a URL to a download possibility:
-    	$query = $_SERVER["QUERY_STRING"];
-	if ($_GET[asciiclean] == 1 or $_COOKIE["asciiclean"] == "clean"){
-		$url = '<center><a href="base_payload.php?'.$_SERVER["QUERY_STRING"].'&download=1&cid='.$cid.'&sid='.$sid.'&asciiclean=1">Download of Payload</a></center>';
+    	$query = CleanVariable($_SERVER["QUERY_STRING"], VAR_PERIOD | VAR_DIGIT | VAR_PUNC | VAR_LETTER);
+	if ( ($_GET[asciiclean] == 1) || ( ($_COOKIE["asciiclean"] == "clean") && (!isset($_GET[asciiclean])) ) ){
+		$url = '<center><a href="base_payload.php?'.$query;
+		$url.= '&download=1&cid='.$cid.'&sid='.$sid.'&asciiclean=1">Download of Payload</a></center>';
 	} else {
-		$url = '<center><a href="base_payload.php?'.$_SERVER["QUERY_STRING"].'&download=1&cid='.$cid.'&sid='.$sid.'&asciiclean=0">Download of Payload</a></center>';
+		$url = '<center><a href="base_payload.php?'.$query;
+		$url.= '&download=1&cid='.$cid.'&sid='.$sid.'&asciiclean=0">Download of Payload</a></center>';
+	}
+	return $url;
+}
+
+function PrintPcapDownload($db, $cid, $sid){
+// If we have FLoP extended database offer a URL to download in pcap format:
+
+        if ( !in_array("pcap_header", $db->DB->MetaColumnNames('data')) ||
+             !in_array("data_header", $db->DB->MetaColumnNames('data'))) {
+		return;
+	}
+
+    	$query = CleanVariable($_SERVER["QUERY_STRING"], VAR_PERIOD | VAR_DIGIT | VAR_PUNC | VAR_LETTER);
+	if ( ($_GET[asciiclean] == 1) || ( ($_COOKIE["asciiclean"] == "clean") && (!isset($_GET[asciiclean])) ) ){
+		$url = '<center><a href="base_payload.php?'.$query;
+		$url.= '&download=2&cid='.$cid.'&sid='.$sid.'&asciiclean=1">Download in pcap format</a></center>';
+	} else {
+		$url = '<center><a href="base_payload.php?'.$query;
+		$url.= '&download=2&cid='.$cid.'&sid='.$sid.'&asciiclean=0">Download in pcap format</a></center>';
 	}
 	return $url;
 }
@@ -99,11 +120,14 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
 
      if ( $myrow2 == "" )
         $next_button = '[ '._LAST.' ]'."\n"; 
-     else if ( $i == $seq-1 )
-        $previous_button = '<INPUT TYPE="submit" NAME="submit" VALUE="&lt;&lt; '._PREVIOUS.' #'.($seq-1).'-('.$myrow2[0].'-'.$myrow2[1].')">'."\n";
-
-     else if ( $i == $seq+1 )
-        $next_button = '<INPUT TYPE="submit" NAME="submit" VALUE="&gt;&gt; '._NEXT.' #'.($seq+1).'-('.$myrow2[0].'-'.$myrow2[1].')">'."\n";
+     else if ( $i == $seq-1 ) {
+        $previous_button = '<INPUT TYPE="submit" NAME="submit" VALUE="&lt;&lt; '._PREVIOUS.' #';
+        $previous_button.= ($seq-1).'-('.$myrow2[0].'-'.$myrow2[1].')">'."\n";
+     }
+     else if ( $i == $seq+1 ) {
+        $next_button = '<INPUT TYPE="submit" NAME="submit" VALUE="&gt;&gt; '._NEXT.' #';
+        $next_button.= ($seq+1).'-('.$myrow2[0].'-'.$myrow2[1].')">'."\n";
+     }
      $i++;
   }
 
@@ -115,7 +139,10 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
    *  the back button.  Otherwise, the value of $submit will not be passed
    *  to the history.
    */
-  $submit = ImportHTTPVar("submit");
+
+  /* This call can include "#xx-(xx-xx)" values and "submit" values. */
+  $submit = ImportHTTPVar("submit", VAR_DIGIT | VAR_PUNC | VAR_LETTER, array(_SELECTED, _ALLONSCREEN, _ENTIREQUERY)); 
+
   $_SERVER["QUERY_STRING"] = "submit=".rawurlencode($submit);
 
   $et = new EventTiming($debug_time_mode);
@@ -125,7 +152,7 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
   $qs = new QueryState();
 
   $page_title = _ALERT;
-  PrintBASESubHeader($page_title, $page_title, $cs->GetBackLink());
+  PrintBASESubHeader($page_title, $page_title, $cs->GetBackLink(), 1);
 
   /* Connect to the Alert database */
   $db = NewBASEDBConnection($DBlib_path, $DBtype);
@@ -161,7 +188,7 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
    * get the (sid,cid) back from $caller 
    */ 
   if ( $submit == _SELECTED )
-     $submit = ImportHTTPVar("caller");
+     $submit = ImportHTTPVar("caller", VAR_DIGIT | VAR_PUNC);
   else
      $caller = $submit;
 
@@ -209,7 +236,7 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
      exit();
   }
 
-  echo "<FORM METHOD=\"POST\" ACTION=\"base_qry_alert.php\">\n"; 
+  echo "<FORM METHOD=\"GET\" ACTION=\"base_qry_alert.php\">\n"; 
   PrintPacketLookupBrowseButtons($seq, $save_sql, $db, $previous, $next);
   echo "<CENTER>\n<B>"._ALERT." #".($seq)."</B><BR>\n$previous &nbsp&nbsp&nbsp\n$next\n</CENTER>\n";
   echo "<HR>\n";
@@ -229,6 +256,14 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
      echo '</CENTER></B>';
   }
 
+  /* Get sensor parameters: */
+  $sql4 = "SELECT hostname, interface, filter, encoding, detail FROM sensor  WHERE sid='".$sid."'";
+  $result4 = $db->baseExecute($sql4);
+  $myrow4 = $result4->baseFetchRow();
+  $result4->baseFreeRows();
+  $encoding = $myrow4[3];
+  $detail = $myrow4[4];
+
   echo '
        <BLOCKQUOTE>
        <TABLE BORDER=1 width="90%">
@@ -240,15 +275,11 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
                         <TD CLASS="plfieldhdr">'._QATRIGGERSIG.'</TD></TR>
                     <TR><TD CLASS="plfield">'.($sid." - ".$cid).'</TD>
                         <TD CLASS="plfield">'.$myrow2[1].'</TD>
-                        <TD CLASS="plfield">'.BuildSigByID($myrow2[0], $db).'</TD></TR>      
+                        <TD CLASS="plfield">'.GetTagTriger(BuildSigByID($myrow2[0], $db), $db, $sid, $cid).'</TD></TR>      
                   </TABLE>
               </TD>
            </TR>';
 
-  $sql4 = "SELECT hostname, interface, filter FROM sensor  WHERE sid='".$sid."'";
-  $result4 = $db->baseExecute($sql4);
-  $myrow4 = $result4->baseFetchRow();
-
   echo '  <TR>
              <TD>
                 <TABLE BORDER=1 CELLPADDING=4>
@@ -327,7 +358,8 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
                         <TD class="plfieldhdr">TOS</TD>
                         <TD class="plfieldhdr">'._LENGTH.'</TD>
                         <TD class="plfieldhdr">'._ID.'</TD>
-                        <TD class="plfieldhdr">flags</TD>
+                        <TD class="plfieldhdr">D<BR>F</TD>
+                        <TD class="plfieldhdr">M<BR>F</TD>
                         <TD class="plfieldhdr">offset</TD>
                         <TD class="plfieldhdr">TTL</TD>
                         <TD class="plfieldhdr">chksum</TD></TR>';
@@ -338,11 +370,16 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
                          <A HREF="base_stat_ipaddr.php?ip='.baseLong2IP($myrow2[1]).'&netmask=32">'.
                             baseLong2IP($myrow2[1]).'</A></TD>';
   echo '                 <TD class="plfield">'.$myrow2[2].'</TD>';
-  echo '                 <TD class="plfield">'.$myrow2[3].'</TD>';
+  echo '                 <TD class="plfield">'.($myrow2[3] << 2).'</TD>';    /* ihl is in 32 bit words, must be multiplied by 4 to show in bytes */
   echo '                 <TD class="plfield">'.$myrow2[4].'</TD>';
   echo '                 <TD class="plfield">'.$myrow2[5].'</TD>';
   echo '                 <TD class="plfield">'.$myrow2[6].'</TD>';
-  echo '                 <TD class="plfield">'.$myrow2[7].'</TD>';
+  if ( ($myrow2[7] & 2 ) != 0 )
+     echo 'X';
+  echo '                    </TD><TD class="plfield">';
+  if ( ($myrow2[7] & 1 ) != 0 )
+     echo 'X';
+  echo '                    </TD><TD class="plfield">';
   echo '                 <TD class="plfield">'.$myrow2[8].'</TD>';
   echo '                 <TD class="plfield">'.$myrow2[9].'</TD>';
   echo '                 <TD class="plfield">'.$myrow2[10].'</TD></TR>';
@@ -403,6 +440,62 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
   }
   $result2->baseFreeRows();
 
+
+  /* If we have FLoP's (Fast Logging Project for Snort) extended 
+   * database schema then we can show mac addresses from `data_header`
+   * field from `data` table
+   */
+  if (in_array("data_header", $db->DB->MetaColumnNames('data'))) {
+
+     $sql5 = "SELECT data_header FROM data WHERE sid='$sid' AND cid='$cid'";
+     $result5 = $db->baseExecute($sql5);
+     $myrow5 = $result5->baseFetchRow();
+     $result5->baseFreeRows();
+
+     if ( $debug_mode > 0 ) {
+	echo "&lt;debug&gt;<BR>";
+	echo "Encoding: $myrow6[0]<BR>";
+	echo "Data header: &lt;$myrow5[0]&gt;<BR>";
+	echo "strlen: " . strlen($myrow5[0]);
+	echo "<br>Base64 decoded: &lt;" . base64_decode($myrow5[0]) . "&gt;<BR>";
+	echo "strlen: " . strlen(base64_decode($myrow5[0]));
+	echo "<br>bin2hex: &lt;" . bin2hex(base64_decode($myrow5[0])) . "&gt;<BR>";
+	echo "strlen: " . strlen(bin2hex(base64_decode($myrow5[0])));
+	echo "<BR>&lt;/debug&gt;<BR>";
+     }
+
+     /* 0 == hex, 1 == base64, 2 == ascii; cf. snort-2.4.4/src/plugbase.h */
+     if ($encoding == 0) {
+       	$t = $myrow5[0];
+     } elseif ($encoding == 1) {
+       	$t = bin2hex(base64_decode($myrow5[0]));
+     } else {
+        echo "<BR><BR>This type of encoding is not supported. Please use either hex oder ";
+        echo "base64 encoding. Do not use ascii, because ascii encoding loses data.<BR><BR>";
+     }
+
+     /* from here on $t is in hex format, even if original encoding was base64 */
+     if ( strlen($t) >= 24 ) {
+        $dst_mac = $t[0].$t[1].':'.$t[2].$t[3].':'.$t[4].$t[5].':'.$t[6].$t[7].':'.$t[8].$t[9].':'.$t[10].$t[11];
+        $src_mac = $t[12].$t[13].':'.$t[14].$t[15].':'.$t[16].$t[17].':'.$t[18].$t[19].':'.$t[20].$t[21].':'.$t[22].$t[23];
+
+        echo '
+             <TABLE BORDER=1 WIDTH="90%">
+                <TR><TD CLASS="iptitle" WIDTH=50 ROWSPAN=3 ALIGN=CENTER>MAC';
+        echo '      <TD>';
+        echo '         <TABLE BORDER=1 CELLPADDING=2>';
+        echo '            <TR><TD class="plfieldhdr">'._NBSOURCEADDR.'</TD>
+                              <TD class="plfieldhdr">&nbsp;'._NBDESTADDR.'&nbsp</TD></TR>
+                          <TR><TD>'. $src_mac .'</TD>
+                              <TD>'. $dst_mac .'</TD></TR>
+                          <TR><TD>'. GetVendor($src_mac) .'</TD>
+                              <TD>'. GetVendor($dst_mac) .'</TD></TR>';
+        echo '         </TABLE>';
+        echo '</TABLE></TD></TR>';
+     }
+  }
+
+
   /* TCP */
   if ( $layer4_proto == "6" )  
   {
@@ -585,38 +678,46 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
   /* Print the Payload */
   $sql2 = "SELECT data_payload FROM data WHERE sid='".$sid."' AND cid='".$cid."'";
   $result2 = $db->baseExecute($sql2);
+  $myrow2 = $result2->baseFetchRow();
+  $result2->baseFreeRows();
+  $payload = $myrow2[0];
 
-  /* get encoding information and detail_level on the payload */
-  $sql3 = 'SELECT encoding, detail FROM sensor WHERE sid='.$sid;
-  $result3 = $db->baseExecute($sql3);
-  $myrow3 = $result3->baseFetchRow();  
- 
-/* xxx jl */ 
   echo '
         <TABLE BORDER=1 WIDTH="90%">
            <TR><TD class="payloadtitle" WIDTH=50 ROWSPAN=2 ALIGN=CENTER>Payload';
            echo("<br><br>".PrintCleanURL());
            echo("<br>".PrintBinDownload($db, $cid, $sid));
+           echo("<br>".PrintPcapDownload($db, $cid, $sid));
   echo '       <TD>';
 
-  $myrow2 = $result2->baseFetchRow();
-  if ( $myrow2 )
+  if ( $payload )
   {
      /* print the packet based on encoding type */
-     PrintPacketPayload($myrow2[0], $myrow3[0], 1);
+     PrintPacketPayload($payload, $encoding, 1);
 
      if ( $layer4_proto == "1" )
      {
-      /* IF ( ICMP destination unreachable OR ICMP time exceeded ) */
-      if ( $ICMPitype == "3" || $ICMPitype == "11" )
-      {
-          /* IF ( network, host, port unreachable OR frag needed OR network admin prohibited OR filtered) */
-          if ( $ICMPicode == "0" || $ICMPicode == "1" || $ICMPicode == "3" || 
-               $ICMPicode == "4" || 
-               $ICMPicode == "9" || 
-               $ICMPicode == "13" )
+          if ( /* IF ICMP source quench */ 
+               ($ICMPitype == "4" && $ICMPicode == "0") ||
+               /* IF ICMP redirect */
+               ($ICMPitype == "5") ||
+               /* IF ICMP parameter problem */
+               ($ICMPitype == "12" && $ICMPicode == "0") ||
+               /* IF ( network, host, port unreachable OR 
+               frag needed OR network admin prohibited OR filtered) */
+               ($ICMPitype == "3" || $ICMPitype == "11" ) &&
+               $ICMPicode == "0" || $ICMPicode == "1" ||
+               $ICMPicode == "3" || $ICMPicode == "4" || 
+               $ICMPicode == "9" || $ICMPicode == "13" )
           {
-              $work = str_replace("\n","",$myrow2[0]);
+              /* 0 == hex, 1 == base64, 2 == ascii; cf. snort-2.4.4/src/plugbase.h */
+              if ($encoding == 1) {
+                 /* encoding is base64 */
+                 $work = bin2hex(base64_decode(str_replace("\n","",$payload)));
+              } else {
+                 /* assuming that encoding is hex */
+                 $work = str_replace("\n","",$payload);
+              }
 
               /* 
                *  - depending on how the packet logged, 32-bits of NULL padding after
@@ -629,7 +730,13 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
                */
               else if ( ($ICMPitype == "3") && ($ICMPicode == "4") )
                  $offset += 8;
-
+              
+              if ($ICMPitype == "5") {
+                 $gateway = hexdec($work[16+$offset].$work[17+$offset]).".".
+                            hexdec($work[18+$offset].$work[19+$offset]).".".
+                            hexdec($work[20+$offset].$work[20+$offset]).".".
+                            hexdec($work[22+$offset].$work[23+$offset]);
+              }
               $icmp_src = hexdec($work[24+$offset].$work[25+$offset]).".".
                           hexdec($work[26+$offset].$work[27+$offset]).".".
                           hexdec($work[28+$offset].$work[29+$offset]).".".
@@ -646,27 +753,39 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
 
               echo '<TABLE BORDER=1>';
               echo '<TR>';
+              if ( $ICMPitype == "5" ) {
+                 echo '<TD class="plfieldhdr">Gateway IP</TD>';
+                 echo '<TD class="plfieldhdr">Gateway Name</TD>';
+              }
               echo '<TD class="plfieldhdr">Protocol</TD>';
               echo '<TD class="plfieldhdr">Org.Source<BR>IP</TD>';
               echo '<TD class="plfieldhdr">Org.Source<BR>Name</TD>';
-              echo '<TD class="plfieldhdr">Org.Source<BR>Port</TD>';
+              if ( $icmp_proto == "6" || $icmp_proto == "17" )
+                 echo '<TD class="plfieldhdr">Org.Source<BR>Port</TD>';
               echo '<TD class="plfieldhdr">Org.Destination<BR>IP</TD>';
               echo '<TD class="plfieldhdr">Org.Destination<BR>Name</TD>';
-              echo '<TD class="plfieldhdr">Org.Destination<BR>Port</TD>';
+              if ( $icmp_proto == "6" || $icmp_proto == "17" )
+                 echo '<TD class="plfieldhdr">Org.Destination<BR>Port</TD>';
               echo '</TR>';
               echo '<TR>';
+              if ( $ICMPitype == "5" ) {
+                 echo '<TD class="plfield">';
+                 echo '<A HREF="base_stat_ipaddr.php?ip='.$gateway.'&netmask=32" TARGET="_PL_SIP">'.$gateway.'</A></TD>';
+                 echo '<TD class="plfield">'.baseGetHostByAddr($gateway, $db, $dns_cache_lifetime).'</TD>';
+              }
               echo '<TD class="plfield">'.IPProto2Str($icmp_proto).'</TD>';
               echo '<TD class="plfield">';
               echo '<A HREF="base_stat_ipaddr.php?ip='.$icmp_src.'&netmask=32" TARGET="_PL_SIP">'.$icmp_src.'</A></TD>';
               echo '<TD class="plfield">'.baseGetHostByAddr($icmp_src, $db, $dns_cache_lifetime).'</TD>';
-              echo '<TD class="plfield">'.$icmp_src_port.'</TD>';
+              if ( $icmp_proto == "6" || $icmp_proto == "17" )
+                 echo '<TD class="plfield">'.$icmp_src_port.'</TD>';
               echo '<TD class="plfield">';
               echo '<A HREF="base_stat_ipaddr.php?ip='.$icmp_dst.'&netmask=32" TARGET="_PL_DIP">'.$icmp_dst.'</A></TD>';
               echo '<TD class="plfield">'.baseGetHostByAddr($icmp_dst, $db, $dns_cache_lifetime).'</TD>';
-              echo '<TD class="plfield">'.$icmp_dst_port.'</TD>';
+              if ( $icmp_proto == "6" || $icmp_proto == "17" )
+                 echo '<TD class="plfield">'.$icmp_dst_port.'</TD>';
               echo '</TR>';
               echo '</TABLE>';
-             }
          }
      }
   }
@@ -675,26 +794,24 @@ function PrintPacketLookupBrowseButtons($seq, $save_sql, $db, &$previous_button,
      /* Don't have payload so lets print out why by checking the detail level */
 
      /* if have fast detail level */
-     if ( $myrow3[1] == "0" )
+     if ( $detail == "0" )
         echo '<BR> &nbsp <I>'._QANOPAYLOAD.'</I><BR>';
      else
         echo '<BR> &nbsp <I>'._NONE.' </I><BR>';
   }
-  $result3->baseFreeRows();
-  echo '</TABLE></BLOCKQUOTE><P>';
 
-  $result2->baseFreeRows();
+  echo '</TABLE></BLOCKQUOTE><P>';
 
   echo "<CENTER>$previous &nbsp&nbsp&nbsp $next</CENTER>";
 
   $qs->PrintAlertActionButtons();
   $qs->SaveState();
   ExportHTTPVar("caller", $caller);
-  $et->Mark("Get Query Elements");
-  $et->PrintTiming();
 
   echo "\n</FORM>\n";
   
   PrintBASESubFooter();
 
+  $et->Mark("Get Query Elements");
+  $et->PrintTiming();
 ?>
