@@ -49,6 +49,12 @@ function IsValidActionOp($action_op, $valid_action_op)
        - 2: from signature/alert page
        - 3: from sensor page
        - 4: from AG maintenance page
+       - 5: base_qry_alert.php	PAGE_ALERT_DISPLAY
+       - 6: base_stat_iplink.php	PAGE_STAT_IPLINK
+       - 7: base_stat_class.php	PAGE_STAT_CLASS
+       - 8: base_stat_uaddr.php	PAGE_STAT_UADDR
+       - 9: base_stat_ports.php	PAGE_STAT_PORTS
+ 
   = $action_chk_lst: (used only when _SELECTED is the $action_op)
                     a sparse array where each element contains a key to alerts which should be acted 
                     on.  Some elements will be blank based on the checkbox state.
@@ -286,6 +292,7 @@ function ProcessSelectedAlerts($action, &$action_op, $action_arg, $action_param,
   else if ($action == "del_alert")      $action_desc = _DELETEALERT;
   else if ($action == "email_alert")    $action_desc = _EMAILALERTSFULL;
   else if ($action == "email_alert2")   $action_desc = _EMAILALERTSSUMM;
+  else if ($action == "csv_alert")      $action_desc = _EMAILALERTSCSV;
   else if ($action == "clear_alert")    $action_desc = _CLEARAG;
   else if ($action == "archive_alert")  $action_desc = _ARCHIVEALERTSCOPY;
   else if ($action == "archive_alert2") $action_desc = _ARCHIVEALERTSMOVE;
@@ -382,10 +389,74 @@ function ProcessSelectedAlerts($action, &$action_op, $action_arg, $action_param,
             $sql = "SELECT sid, cid FROM acid_event WHERE sid='".$tmp."'";  
             $sql2 = "SELECT count(sid) FROM acid_event WHERE sid='".$tmp."'"; 
          }
+         /* Unique Classification listing */
+         else if ( $context == PAGE_STAT_CLASS )
+         {
+            if ( !isset($action_lst[$j]) ) $tmp = -1;  else $tmp = $action_lst[$j];
+            $sql = "SELECT acid_event.sid, acid_event.cid  ".$action_sql." AND sig_class_id='".
+                     $tmp."'";  
+            $sql2 = "SELECT count(acid_event.sid) ".$action_sql." AND sig_class_id='".
+                     $tmp."'"; 
+         }
+         /* Unique IP links listing */
+         else if ( $context == PAGE_STAT_IPLINK )
+         {
+            if ( !isset($action_lst[$j]) ) {
+                     $tmp = -1;
+            } else {
+                     $tmp = $action_lst[$j];
+		     $tmp_sip = strtok($tmp, "_");
+		     $tmp_dip = strtok("_");
+		     $tmp_proto = strtok("_");
+		     $tmp = $tmp_sip . "' AND ip_dst='" . $tmp_dip . "' AND ip_proto='" . $tmp_proto;
+            }
+            $sql = "SELECT acid_event.sid, acid_event.cid  ".$action_sql." AND ip_src='". $tmp."'";
+            $sql2 = "SELECT count(acid_event.sid) ".$action_sql." AND ip_src='". $tmp."'"; 
+         }
+         /* Unique IP addrs listing */
+         else if ( $context == PAGE_STAT_UADDR )
+         {
+            if ( !isset($action_lst[$j]) ) {
+                     $tmp = "ip_src='-1'";
+            } else {
+                     $tmp = $action_lst[$j];
+                     if ($tmp[0] != "_")
+                           $tmp_sip = substr($tmp, 0, strlen($tmp)-1);
+                     else
+                           $tmp_dip = substr($tmp, 1, strlen($tmp)-1);
+		     ($tmp_sip != "") ? ($tmp = "ip_src='".$tmp_sip."'") : ($tmp = "ip_dst='".$tmp_dip."'");
+            }
+            $sql = "SELECT acid_event.sid, acid_event.cid  ".$action_sql." AND ". $tmp;
+            $sql2 = "SELECT count(acid_event.sid) ".$action_sql." AND ". $tmp; 
+         }
+         /* Ports listing */
+         else if ( $context == PAGE_STAT_PORTS )
+         {
+            if ( !isset($action_lst[$j]) ) {
+                     $tmp = "ip_proto='-1'";
+            } else {
+                     $tmp = $action_lst[$j];
+                     $tmp_proto = strtok($tmp, "_");
+                     $tmp_porttype = strtok("_");
+                     $tmp_ip = strtok("_");
+                     if ($proto == TCP)
+                           $tmp = "ip_proto='". TCP ."'";
+                     else if ($proto == UDP)
+                           $tmp = "ip_proto='". UDP ."'";
+                     else
+                           $tmp = "ip_proto IN (". TCP .", ". UDP.")";
+                     ($tmp_porttype == SOURCE_PORT) ?
+                           ($tmp.=" AND layer4_sport='".$tmp_ip."'") : ($tmp.=" AND layer4_dport='".$tmp_ip."'");
+            }
+            $sql = "SELECT acid_event.sid, acid_event.cid FROM acid_event WHERE ". $tmp;
+            $sql2 = "SELECT count(acid_event.sid) FROM acid_event WHERE ". $tmp; 
+         }
          /* if acting on alerts by signature or sensor, count the 
           * the number of alerts 
           */
-         if ( ($context == PAGE_STAT_ALERTS) || ($context == PAGE_STAT_SENSOR) )
+         if ( ($context == PAGE_STAT_ALERTS) || ($context == PAGE_STAT_SENSOR)
+            || ($context == PAGE_STAT_CLASS) || ($context == PAGE_STAT_IPLINK)
+            || ($context == PAGE_STAT_UADDR) || ($context == PAGE_STAT_PORTS) )
          {
             $result2 = $db->baseExecute($sql2);
             $myrow2 = $result2->baseFetchRow();
@@ -471,7 +542,9 @@ function ProcessSelectedAlerts($action, &$action_op, $action_arg, $action_param,
      *  Print different message if alert action units (e.g. sensor
      *  or signature) are not individual alerts
      */ 
-    if ( ($context == PAGE_STAT_ALERTS) || ($context == PAGE_STAT_SENSOR) )
+    if ( ($context == PAGE_STAT_ALERTS) || ($context == PAGE_STAT_SENSOR)
+       || ($context == PAGE_STAT_CLASS) || ($context == PAGE_STAT_IPLINK)
+       || ($context == PAGE_STAT_UADDR) || ($context == PAGE_STAT_PORTS) )
         ErrorMessage(_SUCCESS ." $action_desc - "._ON." $action_cnt "._ALERTSPARA." ("._IN." $num_alert_blobs blobs)");
      else
         ErrorMessage(_SUCCESS ." $action_desc - ".$action_cnt._ALERTSPARA);
@@ -649,13 +722,23 @@ function Action_del_alert_op($sid, $cid, $db, $action_arg, &$ctx)
 
 function Action_del_alert_post($action_arg, &$action_ctx, $db, &$num_alert, $action_cnt, $context)
 {
-  /* If unit of delete (i.e. by sensor, signature) is not individual alerts, 
-   * decrement by units not individual alerts 
-   */
-  if ( ($context == 2) || ($context == 3) )
-    $num_alert -= count(ImportHTTPVar("action_chk_lst"));
-  else
-    $num_alert -= $action_cnt;
+  $sel_cnt = 0;
+  $action_lst_cnt = count(ImportHTTPVar("action_lst"));
+  $action_chk_lst = ImportHTTPVar("action_chk_lst");
+
+  /* count the number of check boxes selected  */
+  for ( $i = 0; $i < $action_lst_cnt ; $i++)  
+  { 
+     if ($action_chk_lst[$i] > "" )
+       $sel_cnt += 1;
+  }
+  if ($sel_cnt > 0)       /* 1 or more check boxes selected ? */
+      $num_alert -= $sel_cnt;
+ /* No, must have been a Delete ALL on Screen or Delete Entire Query  */
+  elseif  ($context == 1)     /* detail alert list ? */
+      $num_alert -= $action_cnt; 
+  else  
+      $num_alert -= count(ImportHTTPVar("action_chk_lst"));  
 
 }
 
@@ -681,6 +764,10 @@ function Action_email_alert_post($action_arg, &$action_ctx, $db, &$num_alert, $a
   GLOBAL $BASE_VERSION, 
          $action_email_from, $action_email_mode, $action_email_subject, $action_email_msg;
 
+  /* Return if there is no alerts */
+  if ( $action_ctx == "" )
+        return;
+
   $mail_subject = $action_email_subject;
   $mail_content = $action_email_msg.
                   _GENBASE . " v$BASE_VERSION on ".date("r",time())."\n";
@@ -822,7 +909,7 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
 
    $db2 = &$ctx;
 
-   $insert_sql = array(20);
+   $insert_sql = array();
    $sql_cnt = 0;
    $archive_cnt = 0;
 
@@ -830,12 +917,19 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
           "WHERE sid=$sid";
    $tmp_result = $db->baseExecute($sql);
    $tmp_row = $tmp_result->baseFetchRow();
+   $tmp_result->baseFreeRows();
 
-   if ( $tmp_row )
+   /* Run the same query on archive db, to check if sensor data already in */
+   $tmp_result_db2 = $db2->baseExecute($sql);
+   $tmp_row_db2 = $tmp_result_db2->baseFetchRow();
+   $tmp_result_db2->baseFreeRows();
+
+   /* Insert sensor data only if we got it from alerts db and it's not already in archive db */
+   if ( $tmp_row && !$tmp_row_db2 )
    {
-      $sql = "INSERT INTO sensor (sid,hostname,interface,filter,detail,encoding) ".
+      $sql = "INSERT INTO sensor (sid,hostname,interface,filter,detail,encoding,last_cid) ".
              "VALUES ($sid,'".$tmp_row[0]."','".$tmp_row[1]."','".$tmp_row[2]."','".
-             $tmp_row[3]."','".$tmp_row[4]."')";
+             $tmp_row[3]."','".$tmp_row[4]."','0')";
       if ($db->DB_type == "mssql")
       {
          $insert_sql[$sql_cnt++] = "SET IDENTITY_INSERT sensor ON";
@@ -845,21 +939,40 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
       {
          $insert_sql[$sql_cnt++] = "SET IDENTITY_INSERT sensor OFF";
       }
-      $tmp_result->baseFreeRows();
    }
 
-   $sql = "SELECT signature, timestamp FROM event WHERE sid=$sid AND cid=$cid";
+   /* If we have FLoP's event `reference` column - archive it too. */
+   if ( in_array("reference", $db->DB->MetaColumnNames('event')) ) {
+      $sql = "SELECT signature, timestamp, reference FROM event WHERE sid=$sid AND cid=$cid";
+   } else
+      $sql = "SELECT signature, timestamp FROM event WHERE sid=$sid AND cid=$cid";
+
    $tmp_result = $db->baseExecute($sql);   
    $tmp_row = $tmp_result->baseFetchRow();
    $sig = $tmp_row[0];
    $timestamp = $tmp_row[1];
+   $reference = $tmp_row[2];	/* FLoP's event reference */
    $tmp_result->baseFreeRows();
 
+   /* Run the same query on archive db, to check if event data already in */
+   $tmp_result_db2 = $db2->baseExecute($sql);
+   $tmp_row_event_db2 = $tmp_result_db2->baseFetchRow();
+   $tmp_result_db2->baseFreeRows();
+
    $sig_name = "";
-   if ( $db->baseGetDBversion() < 100 )
+
+   /* Insert event data only if we got it from alerts db and it's not already in archive db */
+   if ( $db->baseGetDBversion() < 100 && !$tmp_row_event_db2)
    {
-      $sql = "INSERT INTO event (sid,cid,signature,timestamp) VALUES ".
-             "($sid, $cid, '".$sig."', '".$timestamp."')"; 
+      /* If we have FLoP's event `reference` column - archive it too. */
+      if ($reference != "") {
+         $sql = "INSERT INTO event (sid,cid,signature,timestamp,reference) VALUES ";
+         $sql.= "($sid, $cid, '".$sig."', '".$timestamp."', '".$reference."')"; 
+      } else {
+         $sql = "INSERT INTO event (sid,cid,signature,timestamp) VALUES ";
+         $sql.= "($sid, $cid, '".$sig."', '".$timestamp."')"; 
+      }
+
       $insert_sql[$sql_cnt++] = $sql;
    }
    /* Catch alerts with a null signature (e.g. with use of tag rule option) */
@@ -868,16 +981,22 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
       $sig_name = GetSignatureName($sig, $db);
 
       if ( $db->baseGetDBversion() >= 103 )
-      {
-         $result = $db->baseExecute("SELECT sig_class_id, sig_priority,
-                                     sig_rev, sig_sid FROM signature
-                                     WHERE sig_id = '".$sig."'");
+      {  
+         if ( $db->baseGetDBversion() >= 107 )
+            $sql = "SELECT sig_class_id, sig_priority, sig_rev, sig_sid, sig_gid ";
+         else
+            $sql = "SELECT sig_class_id, sig_priority, sig_rev, sig_sid ";
+
+         $sql .= "FROM signature WHERE sig_id = '".$sig."'";
+         $result = $db->baseExecute($sql);
          $row = $result->baseFetchRow();
          $sig_class_id = $row[0];
          $sig_class_name = GetSigClassName($sig_class_id, $db);
          $sig_priority = $row[1];
          $sig_rev = $row[2];
          $sig_sid = $row[3];
+         if ( $db->baseGetDBversion() >= 107 )
+            $sig_gid = $row[4];
       }
 
       $MAX_REF_CNT = 6;
@@ -895,11 +1014,11 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
                 "WHERE ref_id='".$ref_id."'";
          $tmp_result2 = $db->baseExecute($sql);
          $tmp_row2 = $tmp_result2->baseFetchRow();   
+         $tmp_result2->baseFreeRows();
 
          $sig_reference[$sig_reference_cnt++] = array ($tmp_row2[0],
                                                        $tmp_row2[1],
                                                        GetRefSystemName($tmp_row2[0], $db));
-         $tmp_result2->baseFreeRows();
       }
       $tmp_result->baseFreeRows();
 
@@ -918,21 +1037,31 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
           "FROM iphdr WHERE sid='$sid' AND cid='$cid'";
    $tmp_result = $db->baseExecute($sql);   
    $tmp_row = $tmp_result->baseFetchRow();
+   $tmp_result->baseFreeRows();
+
+   /* Run the same query on archive db, to check if iphdr data already in */
+   $tmp_result_db2 = $db2->baseExecute($sql);
+   $tmp_row_db2 = $tmp_result_db2->baseFetchRow();
+   $tmp_result_db2->baseFreeRows();
+
+   /* Insert iphdr data only if we got it from alerts db */
    if ( $tmp_row )
    {
-      $sql = "INSERT INTO iphdr (sid,cid,
-                              ip_src,
-                              ip_dst,
-                              ip_ver,ip_hlen,ip_tos,ip_len,ip_id,ip_flags,
-                              ip_off,ip_ttl,ip_proto,ip_csum) VALUES ".
-          "($sid, $cid, '".$tmp_row[0]."', '".$tmp_row[1]."',".
-          "'".$tmp_row[2]."','".$tmp_row[3]."','".$tmp_row[4]."','".$tmp_row[5]."',".
-          "'".$tmp_row[6]."','".$tmp_row[7]."','".$tmp_row[8]."','".$tmp_row[9]."',".
-          "'".$tmp_row[10]."','".$tmp_row[11]."')";
-      $tmp_result->baseFreeRows();
-      $insert_sql[$sql_cnt++] = $sql;
-
       $ip_proto = $tmp_row[10];
+
+      /* Insert iphdr data only if it's not already in archive db */
+      if (!$tmp_row_db2) {
+         $sql = "INSERT INTO iphdr (sid,cid,
+                                 ip_src,
+                                 ip_dst,
+                                 ip_ver,ip_hlen,ip_tos,ip_len,ip_id,ip_flags,
+                                 ip_off,ip_ttl,ip_proto,ip_csum) VALUES ".
+             "($sid, $cid, '".$tmp_row[0]."', '".$tmp_row[1]."',".
+             "'".$tmp_row[2]."','".$tmp_row[3]."','".$tmp_row[4]."','".$tmp_row[5]."',".
+             "'".$tmp_row[6]."','".$tmp_row[7]."','".$tmp_row[8]."','".$tmp_row[9]."',".
+             "'".$tmp_row[10]."','".$tmp_row[11]."')";
+         $insert_sql[$sql_cnt++] = $sql;
+      }
    }
    else
       $ip_proto = -1;
@@ -944,15 +1073,24 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
              "FROM tcphdr WHERE sid='$sid' AND cid='$cid'";
       $tmp_result = $db->baseExecute($sql);   
       $tmp_row = $tmp_result->baseFetchRow();
-      $sql = "INSERT INTO tcphdr (sid,cid,
+      $tmp_result->baseFreeRows();
+
+      /* Run the same query on archive db, to check if tcphdr data already in */
+      $tmp_result_db2 = $db2->baseExecute($sql);
+      $tmp_row_db2 = $tmp_result_db2->baseFetchRow();
+      $tmp_result_db2->baseFreeRows();
+       
+      /* Insert tcphdr data only if we got it from alerts db and it's not already in archive db */
+      if ( $tmp_row && !$tmp_row_db2 ) {
+          $sql = "INSERT INTO tcphdr (sid,cid,
                                tcp_sport, tcp_dport, tcp_seq,
                                tcp_ack, tcp_off, tcp_res, tcp_flags,
                                tcp_win, tcp_csum, tcp_urp) VALUES ".
-          "($sid, $cid, '".$tmp_row[0]."', '".$tmp_row[1]."',".
-          "'".$tmp_row[2]."','".$tmp_row[3]."','".$tmp_row[4]."','".$tmp_row[5]."',".
-          "'".$tmp_row[6]."','".$tmp_row[7]."','".$tmp_row[8]."','".$tmp_row[9]."')";
-      $tmp_result->baseFreeRows();
-      $insert_sql[$sql_cnt++] = $sql;
+             "($sid, $cid, '".$tmp_row[0]."', '".$tmp_row[1]."',".
+             "'".$tmp_row[2]."','".$tmp_row[3]."','".$tmp_row[4]."','".$tmp_row[5]."',".
+             "'".$tmp_row[6]."','".$tmp_row[7]."','".$tmp_row[8]."','".$tmp_row[9]."')";
+          $insert_sql[$sql_cnt++] = $sql;
+      }
    }
    else if ( $ip_proto == 17 )
    {
@@ -960,12 +1098,21 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
              "FROM udphdr WHERE sid='$sid' AND cid='$cid'";
       $tmp_result = $db->baseExecute($sql);   
       $tmp_row = $tmp_result->baseFetchRow();
-      $sql = "INSERT INTO udphdr (sid,cid, udp_sport, udp_dport, ".
-             "udp_len, udp_csum) VALUES ".
-             "($sid, $cid, '".$tmp_row[0]."', '".$tmp_row[1]."',".
-             "'".$tmp_row[2]."','".$tmp_row[3]."')";
       $tmp_result->baseFreeRows();
-      $insert_sql[$sql_cnt++] = $sql;
+
+      /* Run the same query on archive db, to check if udphdr data already in */
+      $tmp_result_db2 = $db2->baseExecute($sql);
+      $tmp_row_db2 = $tmp_result_db2->baseFetchRow();
+      $tmp_result_db2->baseFreeRows();
+          
+      /* Insert udphdr data only if we got it from alerts db and it's not already in archive db */
+      if ( $tmp_row && !$tmp_row_db2 ) {
+         $sql = "INSERT INTO udphdr (sid,cid, udp_sport, udp_dport, ".
+                "udp_len, udp_csum) VALUES ".
+                "($sid, $cid, '".$tmp_row[0]."', '".$tmp_row[1]."',".
+                "'".$tmp_row[2]."','".$tmp_row[3]."')";
+         $insert_sql[$sql_cnt++] = $sql;
+      }
    }
    else if ( $ip_proto == 1 )
    {
@@ -973,39 +1120,86 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
              "FROM icmphdr WHERE sid='$sid' AND cid='$cid'";
       $tmp_result = $db->baseExecute($sql);   
       $tmp_row = $tmp_result->baseFetchRow();
-      $sql = "INSERT INTO icmphdr (sid,cid,icmp_type,icmp_code,".
-             "icmp_csum,icmp_id,icmp_seq) VALUES ".
-             "($sid, $cid, '".$tmp_row[0]."', '".$tmp_row[1]."',".
-             "'".$tmp_row[2]."','".$tmp_row[3]."','".$tmp_row[4]."')";
       $tmp_result->baseFreeRows();
-      $insert_sql[$sql_cnt++] = $sql;
+
+      /* Run the same query on archive db, to check if icmphdr data already in */
+      $tmp_result_db2 = $db2->baseExecute($sql);
+      $tmp_row_db2 = $tmp_result_db2->baseFetchRow();
+      $tmp_result_db2->baseFreeRows();
+      
+      /* Insert icmphdr data only if we got it from alerts db and it's not already in archive db */
+      if ( $tmp_row && !$tmp_row_db2 ) {
+         $sql = "INSERT INTO icmphdr (sid,cid,icmp_type,icmp_code,".
+                "icmp_csum,icmp_id,icmp_seq) VALUES ".
+                "($sid, $cid, '".$tmp_row[0]."', '".$tmp_row[1]."',".
+                "'".$tmp_row[2]."','".$tmp_row[3]."','".$tmp_row[4]."')";
+         $insert_sql[$sql_cnt++] = $sql;
+      }
    }
 
-   $sql = "SELECT data_payload FROM data WHERE sid='$sid' AND cid='$cid'";
-   $tmp_result = $db->baseExecute($sql);   
-   $tmp_row = $tmp_result->baseFetchRow();
-   if ( $tmp_row )
+   /* If we have FLoP extended db, archive `pcap_header` and `data_header` too. */
+   if ( in_array("pcap_header", $db->DB->MetaColumnNames('data')) &&
+        in_array("data_header", $db->DB->MetaColumnNames('data'))) {
+      $sql = "SELECT data_payload, pcap_header, data_header FROM data WHERE sid='$sid' AND cid='$cid'";
+      $tmp_result = $db->baseExecute($sql);   
+      $tmp_row = $tmp_result->baseFetchRow();
+      $tmp_result->baseFreeRows();
+      $pcap_header = $tmp_row[1];
+      $data_header = $tmp_row[2];
+   } else { 
+      $sql = "SELECT data_payload FROM data WHERE sid='$sid' AND cid='$cid'";
+      $tmp_result = $db->baseExecute($sql);   
+      $tmp_row = $tmp_result->baseFetchRow();
+      $tmp_result->baseFreeRows();
+   }
+
+
+   /* Run the same query on archive db, to check if data already in */
+   $tmp_result_db2 = $db2->baseExecute($sql);
+   $tmp_row_db2 = $tmp_result_db2->baseFetchRow();
+   $tmp_result_db2->baseFreeRows(); 
+
+   /* Insert data only if we got it from alerts db and it's not already in archive db */
+   if ( $tmp_row && !$tmp_row_db2 )
    {
-      $sql = "INSERT INTO data (sid,cid, data_payload) VALUES ".
-             "($sid, $cid, '".$tmp_row[0]."')";
-      $tmp_result->baseFreeRows(); 
+
+      /* If we have FLoP extended db `pcap_header` or `data_header` then archive it too. */
+      if ( ($pcap_header!="") || ($data_header!="") ) {
+         $sql = "INSERT INTO data (sid,cid, data_payload, pcap_header, data_header) VALUES ";
+         $sql.= "($sid, $cid, '".$tmp_row[0]."', '".$pcap_header."', '".$data_header."')";
+      } else {
+         $sql = "INSERT INTO data (sid,cid, data_payload) VALUES ";
+         $sql.= "($sid, $cid, '".$tmp_row[0]."')";
+      }
+
       $insert_sql[$sql_cnt++] = $sql;
    }
 
    $sql = "SELECT optid, opt_proto, opt_code, opt_len, opt_data ".
           "FROM opt WHERE sid='$sid' AND cid='$cid'";
    $tmp_result = $db->baseExecute($sql);
+   
 
-   while ( (($tmp_row = $tmp_result->baseFetchRow()) != "") 
-           && ($opt_cnt < 19) )
+   while ( ($tmp_row = $tmp_result->baseFetchRow()) != "" )
    {
       $sql = "INSERT INTO opt (sid,cid,optid,opt_proto,".
              "opt_code,opt_len,opt_data) VALUES ".
              "($sid, $cid, '".$tmp_row[0]."', '".$tmp_row[1]."',".
               "'".$tmp_row[2]."','".$tmp_row[3]."','".$tmp_row[4]."')";
-      
-     $tmp_result->baseFreeRows();
-     $insert_sql[$sql_cnt++] = $sql;
+
+      $select_sql = "SELECT optid, opt_proto, opt_code, opt_len, opt_data ".
+                    "FROM opt WHERE sid='$sid' AND cid='$cid' AND optid='$tmp_row[0]'";
+
+      /* Run the select query on archive db, to check if data already in */
+      $tmp_result_db2 = $db2->baseExecute($select_sql);
+      $tmp_row_db2 = $tmp_result_db2->baseFetchRow();
+      $tmp_result_db2->baseFreeRows(); 
+
+      /* Insert data only if it's not already in archive db */
+      if (!$tmp_row_db2 )
+         $insert_sql[$sql_cnt++] = $sql;
+
+      $tmp_result->baseFreeRows();
    }
 
   $archive_cnt = 0;
@@ -1046,6 +1240,16 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
                          " VALUES ('".$sig_class_name."')";
                   $db2->baseExecute($sql);
                   $sig_class_id = $db2->baseInsertID();
+
+		  /* Kludge query. Getting insert ID fails on postgres. */
+		  if ($db->DB_type == "postgres")
+		  {
+		      $sql = "SELECT last_value FROM sig_class_sig_class_id_seq";
+		      $tmp_result = $db2->baseExecute($sql);
+		      $tmp_row = $tmp_result->baseFetchRow();
+		      $tmp_result->baseFreeRows();
+		      $sig_class_id=$tmp_row[0];
+		  }
               }
               else
               {
@@ -1054,17 +1258,30 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
            }
 
            if ( $sig_priority == "" )  $sig_priority = 'NULL';
+           if ( $sig_rev == "" )  $sig_rev = 'NULL';
            
-           $sql = "INSERT INTO signature ".
-                  "(sig_name, sig_class_id, sig_priority, sig_rev, sig_sid) ". 
-                  "VALUES ('$sig_name',".$sig_class_id.", ".$sig_priority.",".
-                  "'".$sig_rev."', '".$sig_sid."')";
+           if ( $db->baseGetDBversion() >= 107 )
+                $sql = "INSERT INTO signature (sig_name, sig_class_id, sig_priority, sig_rev, sig_sid, sig_gid) ".
+                       "VALUES ('$sig_name',".$sig_class_id.", ".$sig_priority.", ".$sig_rev.", ".$sig_sid.", ".$sig_gid.")";
+           else
+                $sql = "INSERT INTO signature (sig_name, sig_class_id, sig_priority, sig_rev, sig_sid) ".
+                       "VALUES ('$sig_name',".$sig_class_id.", ".$sig_priority.", ".$sig_rev.", ".$sig_sid.")";
         }
         else
            $sql = "INSERT INTO signature (sig_name) VALUES ('".$sig_name."')";
 
         $db2->baseExecute($sql);
         $sig_id = $db2->baseInsertID();     
+
+	/* Kludge query. Getting insert ID fails on postgres. */
+	if ($db->DB_type == "postgres")
+	{
+	    $sql = "SELECT last_value FROM signature_sig_id_seq";     
+	    $tmp_result = $db2->baseExecute($sql);
+	    $tmp_row = $tmp_result->baseFetchRow();
+	    $tmp_result->baseFreeRows();
+	    $sig_id=$tmp_row[0];
+	}
      }
 
      /* add reference information */
@@ -1083,6 +1300,16 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
                   " VALUES ('".$sig_reference[$j][2]."')";
            $db2->baseExecute($sql);
            $ref_system_id = $db2->baseInsertID();          
+
+	   /* Kludge query. Getting insert ID fails on postgres. */
+	   if ($db->DB_type == "postgres")
+	   {
+	       $sql = "SELECT last_value FROM reference_system_ref_system_id_seq";
+	       $tmp_result = $db2->baseExecute($sql);
+	       $tmp_row = $tmp_result->baseFetchRow();
+	       $tmp_result->baseFreeRows();
+	       $ref_system_id=$tmp_row[0];
+	   }          
         }
         else
         {
@@ -1120,20 +1347,49 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
                               $sig_reference[$j][1]."')";
            $db2->baseExecute($sql);
            $ref_id = $db2->baseInsertID(); 
+
+	   /* Kludge query. Getting insert ID fails on postgres. */
+	   if ($db->DB_type == "postgres")
+	   {
+	       $sql = "SELECT last_value FROM reference_ref_id_seq";
+	       $tmp_result = $db2->baseExecute($sql);
+	       $tmp_row = $tmp_result->baseFetchRow();
+	       $tmp_result->baseFreeRows();
+	       $ref_id=$tmp_row[0];
+	   } 
         }        
 
         if ( ($ref_id != "") && ($ref_id > 0) )
         {
            $sql = "INSERT INTO sig_reference (sig_id, ref_seq, ref_id) ".
                   "VALUES (".$sig_id.",".($j+1).",".$ref_id.")";
-           
-           $insert_sql[$sql_cnt++] = $sql;
+           $select_sql = "SELECT sig_id FROM sig_reference WHERE sig_id=".$sig_id.
+                         " AND ref_seq=".($j+1);
+
+           /* Run the select query on archive db, to check if data already in */
+           $tmp_result_db2 = $db2->baseExecute($select_sql);
+           $tmp_row_db2 = $tmp_result_db2->baseFetchRow();
+           $tmp_result_db2->baseFreeRows(); 
+
+           /* Insert data only if it's not already in archive db */
+           if (!$tmp_row_db2 )
+              $insert_sql[$sql_cnt++] = $sql;
         }
      }
+     /* Insert event data only if it's not already in archive db */
+     if (!$tmp_row_event_db2) {
 
-     $sql = "INSERT INTO event (sid,cid,signature,timestamp) VALUES ".
-             "($sid, $cid, '".$sig_id."', '".$timestamp."')"; 
-     $insert_sql[$sql_cnt++] = $sql;
+        /* If we have FLoP's event `reference` column - archive it too. */
+        if ($reference != "") {
+           $sql = "INSERT INTO event (sid,cid,signature,timestamp,reference) VALUES ";
+           $sql.= "($sid, $cid, '".$sig_id."', '".$timestamp."', '".$reference."')";
+        } else {
+           $sql = "INSERT INTO event (sid,cid,signature,timestamp) VALUES ";
+           $sql.= "($sid, $cid, '".$sig_id."', '".$timestamp."')";
+        }
+
+        $insert_sql[$sql_cnt++] = $sql;
+     }
   }
 
   if ( $debug_mode > 1 )
@@ -1160,13 +1416,8 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
                          $archive_user, $archive_password);
         }
 
-        /* Sometimes INSERTs will be made for records which
-         * already exist (e.g. sensor or sig_reference table).
-         * When we get such an error, assume that this is ok
-         */
-        if ( strstr($insert_sql[$j], "INSERT INTO sensor") ||
-             strstr($insert_sql[$j], "INSERT INTO sig_reference") ||
-             strstr($insert_sql[$j], "SET IDENTITY_INSERT") )
+        /* When we get such an error, assume that this is ok */
+        if ( strstr($insert_sql[$j], "SET IDENTITY_INSERT") )
            ++$archive_cnt;
         else
         { 
@@ -1180,11 +1431,18 @@ function Action_archive_alert_op($sid, $cid, &$db, $action_arg, &$ctx)
      }
   } 
 
-  /* Check if all data was written to archive database,
+  /* Check if all or any data was written to archive database,
    * before purging the alert from the current database
    */
-  if ( $archive_cnt == $sql_cnt )
-     $archive_cnt = 1;  
+  if ( ($archive_cnt == $sql_cnt) && ($sql_cnt > 0) ) {
+     $archive_cnt = 1;
+     /*
+      * Update alert cache for archived alert right after we copy it to archive db. 
+      * This fixes issue when alert in archive db not cached if archived alert cid 
+      * is lesser than other alerts cid already cached in archive db.
+      */
+     CacheAlert($sid,$cid,$db2);
+  }
   else
      $archive_cnt = 0;
 
