@@ -95,7 +95,7 @@ function UpdateWhoisCache($db)
 function CacheAlert($sid, $cid, $db)
 {
   $signature = $timestamp = $ip_src = $ip_dst = null;
-  $ip_proto = $sport = $dport = $sig_name = null;
+  $ip_proto = $layer4_sport = $layer4_dport = $sig_name = null;
   $sig_class_id = $sig_priority = null;
 
   $sql = "SELECT signature, timestamp, ip_src, ip_dst, ip_proto FROM event ".
@@ -121,8 +121,8 @@ function CacheAlert($sid, $cid, $db)
         $row = $result->baseFetchRow();
         if ( $row )
         {
-           $sport = $row[0];
-           $dport = $row[1];
+           $layer4_sport = $row[0];
+           $layer4_dport = $row[1];
            $result->baseFreeRows();
         }
      }
@@ -134,8 +134,8 @@ function CacheAlert($sid, $cid, $db)
         $row = $result->baseFetchRow();
         if ( $row )
         {
-           $sport = $row[0];
-           $dport = $row[1];
+           $layer4_sport = $row[0];
+           $layer4_dport = $row[1];
            $result->baseFreeRows();
         }
      }
@@ -168,23 +168,23 @@ function CacheAlert($sid, $cid, $db)
     echo "<PRE>".$sql."</PRE>";
   }
 
-  if ( $db->baseGetDBversion() >= 100 )
-  {
-    $sql = "INSERT INTO acid_event (sid, cid, signature, sig_name, ".
-           "sig_class_id, sig_priority, ".
-           "timestamp, ip_src, ip_dst, ip_proto, layer4_sport, ".
-           "layer4_dport) VALUES ('$sid', '$cid', '$signature', ".
-           "'$sig_name','$sig_class_id','$sig_priority',".
-           "'$timestamp','$ip_src','$ip_dst', ".
-           "'$ip_proto', '$layer4_sport', '$layer4_dport')";
-  }
-  else
-  {
-    $sql = "INSERT INTO acid_event (sid, cid, signature, ".
-           "timestamp, ip_src, ip_dst, ip_proto, layer4_sport, ".
-           "layer4_dport) VALUES ('$sid', '$cid', '$signature', ".
-           "'$timestamp','$ip_src','$ip_dst', ".
-           "'$ip_proto', '$layer4_sport', '$layer4_dport')";
+  /* There can be events without certain attributes */
+  if ($ip_src=='') $ip_src='NULL';
+  if ($ip_dst=='') $ip_dst='NULL';
+  if ($ip_proto=='') $ip_proto='NULL';
+  if ($layer4_sport=='') $layer4_sport='NULL';
+  if ($layer4_dport=='') $layer4_dport='NULL';
+
+  if ( $db->baseGetDBversion() >= 100 ) {
+      $sql = "INSERT INTO acid_event (sid, cid, signature, sig_name, sig_class_id, sig_priority, ";
+      $sql.= "timestamp, ip_src, ip_dst, ip_proto, layer4_sport, layer4_dport) ";
+      $sql.= "VALUES ($sid, $cid, $signature, '$sig_name', $sig_class_id, $sig_priority,";
+      $sql.= "'$timestamp', $ip_src, $ip_dst, $ip_proto, $layer4_sport, $layer4_dport)";
+  } else {
+      $sql = "INSERT INTO acid_event (sid, cid, signature, timestamp, ip_src, ";
+      $sql.= "ip_dst, ip_proto, layer4_sport,layer4_dport) ";
+      $sql.= "VALUES ($sid, $cid, '$signature', '$timestamp', $ip_src, $ip_dst, ";
+      $sql.= "$ip_proto, $layer4_sport, $layer4_dport)";
   }
 
   $db->baseExecute($sql); 
@@ -224,9 +224,9 @@ function CacheSensor($sid, $cid, $db)
 
   /* IP events only */
    if ( $db->baseGetDBversion() >= 100 )
-      $schema_specific[3] = " (sig_name LIKE 'spp_%') ";
+      $schema_specific[3] = " (sig_name LIKE '(spp_%') ";
    else
-      $schema_specific[3] = " (signature LIKE 'spp_%') ";
+      $schema_specific[3] = " (signature LIKE '(spp_%') ";
   
   /* TCP events */
    if( $db->DB_type == 'oci8' ) {
@@ -462,7 +462,7 @@ function UpdateAlertCache($db)
      $ccid_lst->baseFreeRows();
   }  
 
- if ( preg_match("/base_main.php/", $_SERVER['REQUEST_URI']) )
+ if ( preg_match("/base_main.php/", $_SERVER['SCRIPT_NAME']) )
        ErrorMessage(_ADDED.$updated_cache_cnt._ALERTSCACHE, "yellow");
  else
        ErrorMessage(_ADDED.$updated_cache_cnt._ALERTSCACHE);
