@@ -367,7 +367,7 @@ class SignatureCriteria extends SingleElementCriteria
    function SanitizeElement()
    {
       $this->criteria[0] = CleanVariable(@$this->criteria[0], "", array(" ", "=", "LIKE"));
-      $this->criteria[1] = CleanVariable(@$this->criteria[1], VAR_ALPHA | VAR_SPACE);
+      $this->criteria[1] = filterSql(@$this->criteria[1]); /* signature name */
       $this->criteria[2] = CleanVariable(@$this->criteria[2], "", array("=", "!="));
    }
 
@@ -458,7 +458,7 @@ class SignatureClassificationCriteria extends SingleElementCriteria
 
    function SanitizeElement()
    {
-      $this->criteria = CleanVariable($this->criteria, VAR_DIGIT, array("null"));
+      $this->criteria = CleanVariable($this->criteria, VAR_DIGIT);
    }
 
    function PrintForm()
@@ -528,7 +528,7 @@ class SignaturePriorityCriteria extends SingleElementCriteria
    function SanitizeElement()
    {
       $this->criteria[0] = CleanVariable(@$this->criteria[0], "", array("=", "!=", "<", "<=", ">", ">="));
-      $this->criteria[1] = CleanVariable(@$this->criteria[1], VAR_DIGIT, array("null"));
+      $this->criteria[1] = CleanVariable(@$this->criteria[1], VAR_DIGIT);
    }
 
    function PrintForm()
@@ -890,11 +890,11 @@ class IPAddressCriteria extends MultipleElementCriteria
       $this->criteria[$i][0] = CleanVariable($curArr[0], VAR_OPAREN);
       $this->criteria[$i][1] = CleanVariable($curArr[1], "", array_keys($this->valid_field_list));
       $this->criteria[$i][2] = CleanVariable($curArr[2], "", array("=", "!=", "<", "<=", ">", ">="));
-      $this->criteria[$i][3] = CleanVariable($curArr[3], VAR_DIGIT | VAR_PERIOD | VAR_FSLASH);
+      $this->criteria[$i][3] = CleanVariable($curArr[3], VAR_DIGIT);
       $this->criteria[$i][4] = CleanVariable($curArr[4], VAR_DIGIT);
       $this->criteria[$i][5] = CleanVariable($curArr[5], VAR_DIGIT);
       $this->criteria[$i][6] = CleanVariable($curArr[6], VAR_DIGIT);
-      $this->criteria[$i][7] = CleanVariable($curArr[7], VAR_DIGIT);
+      $this->criteria[$i][7] = CleanVariable($curArr[7], VAR_DIGIT | VAR_PERIOD | VAR_FSLASH);
       $this->criteria[$i][8] = CleanVariable($curArr[8], VAR_OPAREN | VAR_CPAREN);
       $this->criteria[$i][9] = CleanVariable($curArr[9], "", array("AND", "OR"));
       // Destroy copy
@@ -1349,7 +1349,7 @@ class Layer4Criteria extends SingleElementCriteria
  
    function SanitizeElement()
    {
-      $this->criteria = CleanVariable($this->criteria, "", array("UDP", "TCP", "ICMP", _NOLAYER4));
+      $this->criteria = CleanVariable($this->criteria, "", array("UDP", "TCP", "ICMP", "RawIP"));
    }
  
    function PrintForm()
